# Copyright 3D Control Systems, Inc. All Rights Reserved 2017-2019.
# Built in San Francisco.

# This software is distributed under a commercial license for personal,
# educational, corporate or any other use.
# The software as a whole or any parts of it is prohibited for distribution or
# use without obtaining a license from 3D Control Systems, Inc.

# All software licenses are subject to the 3DPrinterOS terms of use
# (available at https://www.3dprinteros.com/terms-and-conditions/),
# and privacy policy (available at https://www.3dprinteros.com/privacy-policy/)

import aiohttp
import aiohttp.web

import web_interface.handlers
import web_interface.printers_html
import event_handler_bridge


class WSHandlerBridge(event_handler_bridge.EventHandlerBridge):

    async def websock_handler(self, request):
        self.logger.info('UI websocket connection from: ' + str(request.remote))
        if web_interface.handlers.check_auth(request):
            return aiohttp.web.HTTPUnauthorized()
        else:
            ws = aiohttp.web.WebSocketResponse()
            await ws.prepare(request)
            self.connections.append(ws)
            async for msg in ws:
                if self.app.stop_flag:
                    break
                if msg.type == aiohttp.WSMsgType.TEXT:
                    self.logger.debug("Web socket received: %s" % msg.data)
                    if msg.data == 'close':
                        break
                    elif msg.data == 'get_printers_html':
                        await ws.send_json({'type': 'printers_html', 'data': web_interface.printers_html.get_printers_html(request)})
                    elif msg.data == 'get_update_availability':
                        update_available = request.app['3dp_app'].updater.update_available
                        await ws.send_json({'type': 'update_available', 'data': update_available})
                elif msg.type == aiohttp.WSMsgType.ERROR:
                    self.logger.info('Web socket connection closed with exception %s' % ws.exception())
                    break
                elif msg.type == aiohttp.WSMsgType.CLOSED:
                    self.logger.info('Websocket close')
                    break
            self.connections.remove(ws)
            await ws.close()
            self.logger.info('UI websocket disconnect from: ' + str(request.remote))
            return ws
